﻿'use strict';

// view model
function ChatSummaryViewModel(userProviderKey, operatorMode, selectedSessionId, useStrippedSummary) {
    var self = this;
    self.operatorMode = operatorMode;
    self.isLoading = ko.observable(true);
    self.currentDate = ko.observable();
    self.chatProfiles = null;
    self.virtualMessengerSessions = ko.observable();
    self.internalChatProfiles = ko.observable();
    self.users = ko.observable(null);
    self.activeUsersCount = ko.observable(0);
    self.userProviderKey = userProviderKey;
    self.operatorsOfChatProfile = ko.observable([]);
    self.selectedOperator = ko.observable(null);
    self.chatItems = ko.observableArray([]);
    self.chatItemsByChatProfile = ko.observable({});
    self.selectedSessionId = ko.observable(null);

    self.selectedChatItem = ko.pureComputed(function () {
        if (self.selectedSessionId() === null)
            return null;

        let res = null;
        $(self.chatGuestItems()).each(function (index, chatItem) {
            if (chatItem.sessionId === self.selectedSessionId()) {
                res = chatItem;
                return false;
            }
        });

        return res;
    }, this);

    self.selectedChatItem.subscribe(function (newSelectedChatItem) {
        if (newSelectedChatItem) {
            newSelectedChatItem.getGuestHistoryIfNeeded();
        }
    });

    self.unspottedChatEventsSum = ko.pureComputed(function () {
        var sum = 0;
        $(self.chatGuestItems()).each(function (index, chatItem) {
            let myAttitude = self.getMyAttitude(chatItem.session());
            if (myAttitude === true && chatItem.viewModel()) {
                if (chatItem.sessionId !== self.selectedSessionId())
                    sum += chatItem.viewModel().unspottedEventsCounter();
            }
            else if (myAttitude !== false) {
                let guestUserSession = chatItem.guestUserSession();
                if (guestUserSession && guestUserSession.Session.WaitingMessage)
                    sum += 1;
            }
        });

        return sum;
    }, this);

    self.anyRingingChatMessengerSession = ko.observable(false);
    self.unspottedChatMessengerEventsSum = ko.pureComputed(function () {
        var anyRingingChatMessengerSession = false;
        var sum = 0;
        for (let key in self.virtualMessengerSessions()) {
            if (self.virtualMessengerSessions().hasOwnProperty(key)) {
                let virtualSession = self.virtualMessengerSessions()[key];
                sum += virtualSession.unspottedMessagesCount();

                if (virtualSession.callOptions() && virtualSession.callOptions().isRinging)
                    anyRingingChatMessengerSession = true;
            }
        }

        self.anyRingingChatMessengerSession(anyRingingChatMessengerSession);
        return sum;
    }, this);

    self.canReceiveNewGuests = ko.observable(null);
    self.canReceiveNewGuestsError = ko.observable(false);
    self.setCanReceiveNewGuests = function (canReceiveNewGuests) {
        self.canReceiveNewGuests(null);
        chatHub.server.setCanReceiveNewGuests(canReceiveNewGuests).done(function () {
            self.canReceiveNewGuests(canReceiveNewGuests);
        }).fail(function (error) {
            self.canReceiveNewGuestsError(true);
            console.error('canReceiveNewGuests: ' + error);
        });
    };

    self.guestHistory = {
        summaryId: null,
        summary: ko.observable(null),
        detailId: null,
        detail: ko.observable(null)
    };

    self.clearHistorySummary = function () {
        self.guestHistory.summaryId = null;
        self.guestHistory.summary(null);
        self.guestHistory.detailId = null;
        self.guestHistory.detail(null);
    };

    self.getGuestHistorySummary = function (guestProviderKey) {
        if (self.guestHistory.summaryId !== guestProviderKey) {
            $.getJSON(RQ_URL + "/Chat/GetGuestHistorySummary/" + guestProviderKey).done(function (res) {
                if (!res) {
                    console.warn("No guest history summary result.");
                    return;
                }

                self.guestHistory.summaryId = guestProviderKey;
                self.guestHistory.summary(res);
            }).fail(function (error) {
                console.warn(error);
            });
        }
    };

    self.getGuestHistoryDetail = function (sessionId) {
        if (self.guestHistory.detailId !== sessionId) {
            $.getJSON(RQ_URL + "/Chat/GetGuestHistoryDetail/" + sessionId).done(function (res) {
                if (!res) {
                    console.warn("No guest history detail result.");
                    return;
                }

                self.guestHistory.detailId = sessionId;
                res.userProviderKey = userProviderKey; //aby message templata rozeznala meho a ostatni uzivatele
                var chatProfile = self.getChatProfile(res.ChatProfileKey); //aby message templata zobrazila spravne styly chatu
                res.appearance = ko.observable({
                    OperatorMessageBackgroundColor: chatProfile.OperatorMessageBackgroundColor || "#ECECEC",
                    OperatorMessageTextColor: chatProfile.OperatorMessageTextColor || "#000000",
                    GuestMessageBackgroundColor: chatProfile.GuestMessageBackgroundColor || "#FBFBFB",
                    GuestMessageTextColor: chatProfile.GuestMessageTextColor || "#000000",
                    BorderRadius: chatProfile.BorderRadius || 0
                });

                self.guestHistory.detail(res);
            }).fail(function (error) {
                console.warn(error);
            });
        }
    };

    self.getChatHub = function () {
        return chatHub;
    };

    function ChatOperator(operator)
    {
        var op = this;
        op.userProviderKey = operator.userProviderKey;
        op.fullName = operator.fullName;
        op.canReceiveNewGuests = ko.pureComputed(function () {
            let foundUser = self.users() && self.users()[op.userProviderKey];
            return foundUser ? foundUser().CanReceiveNewGuests : false;
        }, this);

        op.attitudeToSelectedSession = ko.pureComputed(function () {
            var res = undefined;
            var selectedSession = self.selectedChatItem() !== null ? self.selectedChatItem().session() : null;
            if (selectedSession)
            {
                let exp = self.getExpressionOfUser(selectedSession, op.userProviderKey);
                res = exp !== null ? exp.Attitude : null;
            }

            return res;
        }, this);

        op.operatorData = ko.pureComputed(function () {
            return JSON.stringify({
                fullName: op.fullName,
                canReceiveNewGuests: op.canReceiveNewGuests(),
                attitudeToSelectedSession: op.attitudeToSelectedSession()
            });
        }, this);
    }

    self.sendMessage = function (text, sessionId) {
        return new Promise(function (resolve, reject) {
            chatHub.server.addMessage(text, sessionId).done(function () {
                resolve();
            }).fail(function (error) {
                reject(error);
            });
        });
    };

    self.userStatus = ko.observable();
    self.userStatusError = ko.observable(false);
    self.setUserStatus = function (userStatus) {
        self.userStatus(null);
        chatHub.server.setUserStatus(userStatus).done(function () {
            self.userStatus(userStatus);
        }).fail(function (error) {
            self.userStatusError(true);
            console.error('userStatus: ' + error);
        });
    };

    self.setAutoAwayStatus = function (autoAwayTimeout) {
        if ($.connection.hub.state === $.signalR.connectionState.connected) {
            chatHub.server.setAutoAwayStatus(autoAwayTimeout).fail(function (error) {
                console.error('setAutoAwayStatus: ' + error);
            });
        }
    };

    self.chatProfileMaxDate = ko.observable({});

    function setChatItemByChatProfile(chatItem) {
        if (chatItem.chatProfile) {
            let chatProfileKey = chatItem.chatProfile.Key;
            if (!self.chatItemsByChatProfile()[chatProfileKey])
                self.chatItemsByChatProfile()[chatProfileKey] = [];

            var chatItemIndex = self.chatItemsByChatProfile()[chatProfileKey].indexOf(chatItem);
            if (chatItemIndex === -1) {
                if (chatItem.chatProfileDate())
                    self.chatItemsByChatProfile()[chatProfileKey].push(chatItem);
            }
            else {
                if (chatItem.chatProfileDate())
                    self.chatItemsByChatProfile()[chatProfileKey][chatItemIndex] = chatItem;
                else {
                    self.chatItemsByChatProfile()[chatProfileKey].splice(chatItemIndex, 1);
                }
            }

            if (chatItem.chatProfileDate()) {
                self.chatItemsByChatProfile()[chatProfileKey].sort(function compareChatItems(chatItem1, chatItem2) {
                    return chatItem1.chatProfileDate() - chatItem2.chatProfileDate();
                });
            }
        }
    }

    function pairChatItemWithVirtualMessengerSession(chatItem, unpair) {
        if (typeof ChatMessengerItem === 'undefined' || !(chatItem instanceof ChatMessengerItem))
            return;

        let userSessions = chatItem.viewModel().userSessions();
        if (userSessions.length === 1) {
            let virtualMessengerSession = self.virtualMessengerSessions()[userSessions[0].us.User.UserProviderKey];
            if (virtualMessengerSession) {
                if (unpair) {
                    virtualMessengerSession.selectionIsRequired = false;
                    virtualMessengerSession.chatItem(null);
                }
                else {
                    virtualMessengerSession.chatItem(chatItem);
                    if (virtualMessengerSession.selectionIsRequired === true) {
                        virtualMessengerSession.selectionIsRequired = false;
                        if (chatUtils.getIEVersion() === 0) {
                            chatItem.isSelected(true);
                        }
                        else { // IE blbne
                            setTimeout(function () {
                                chatItem.isSelected(true);
                            }, 200);
                        }
                    }
                }
            }
        }
    }

    function delChatItemByChatProfile(chatItem) {
        if (chatItem.chatProfile) {
            let chatProfileKey = chatItem.chatProfile.Key;
            if (self.chatItemsByChatProfile()[chatProfileKey]) {
                var chatItemIndex = self.chatItemsByChatProfile()[chatProfileKey].indexOf(chatItem);
                if (chatItemIndex > -1)
                    self.chatItemsByChatProfile()[chatProfileKey].splice(chatItemIndex, 1);
            }
        }
    }

    function processServerUserSession(sessionId, userSessionObj, promises, checkUpdates) {
        var guestUserSession, userSessions;
        if (!Array.isArray(userSessionObj))
            guestUserSession = userSessionObj;
        else
            userSessions = userSessionObj;

        var chatItemFound = false;
        if (checkUpdates) {
            for (let i = 0; i < self.chatItems().length; i++) {
                let chatItem = self.chatItems()[i];
                if (chatItem.sessionId === sessionId) {
                    chatItemFound = true;
                    if (userSessions && userSessions[0].Session.Disposed || guestUserSession && guestUserSession.Session.Disposed) {
                        pairChatItemWithVirtualMessengerSession(chatItem, true);
                        delChatItemByChatProfile(chatItem);
                        self.chatItems().splice(i--, 1);
                        break;
                    }

                    if (guestUserSession) {
                        chatItem.secludedGuestUserSession(guestUserSession);
                        chatItem.viewModel(null);
                        setChatItemByChatProfile(chatItem);
                    }
                    else {
                        if (chatItem.viewModel()) {
                            chatItem.viewModel().isInited = true;
                            chatItem.viewModel().setUserSessions(userSessions);
                            setChatItemByChatProfile(chatItem);
                            pairChatItemWithVirtualMessengerSession(chatItem);
                        }
                        else {
                            promises.push(chatItem.createAndInitChatSessionViewModel(userSessions).then(function () {
                                setChatItemByChatProfile(chatItem);
                                pairChatItemWithVirtualMessengerSession(chatItem);
                            }));
                        }
                    }

                    break;
                }
            }
        }

        if (!chatItemFound) {
            let newChatItem;
            if (userSessions && !("GuestChatProfileKey" in userSessions[0].Session)) { // messenger session
                if (typeof ChatMessengerItem !== 'undefined')
                    newChatItem = new ChatMessengerItem(userProviderKey, self.currentDate, chatHub, userSessions);
                else return; // chat messenger is not loaded due to some reason (e.g. messenger was turned off when page was loaded)
            }
            else { // guest session
                let chatProfile = self.getChatProfile(guestUserSession ? guestUserSession.Session.GuestChatProfileKey : userSessions[0].Session.GuestChatProfileKey);
                newChatItem = guestUserSession ? new ChatGuestItem(userProviderKey, self.currentDate, chatHub, guestUserSession, chatProfile) : new ChatGuestItem(userProviderKey, self.currentDate, chatHub, userSessions, chatProfile);
            }

            promises.push(newChatItem.init().then(function () {
                self.chatItems().push(newChatItem);
                setChatItemByChatProfile(newChatItem);
                pairChatItemWithVirtualMessengerSession(newChatItem);
            }));
        }
    }

    // operations
    self.setSummary = function (summary) {
        //console.log(summary);

        if (summary.MyUser &&  summary.MyUser.UserProviderKey === userProviderKey) {
            self.userStatus(summary.MyUser.Status);
            self.userStatusError(false);

            if (self.operatorMode) {
                self.canReceiveNewGuests(summary.MyUser.CanReceiveNewGuests);
                self.canReceiveNewGuestsError(false);
            }
        }


        if (summary.ActiveUsersCount !== null)
            self.activeUsersCount(summary.ActiveUsersCount);

        if (summary.Users) {
            if (summary.InitDateTime)
                self.users({});

            if (self.users() !== null) {
                let anyPushedUser = false;
                $(summary.Users).each(function (index, sumUser) {
                    let foundUser = self.users()[sumUser.UserProviderKey];
                    if (foundUser) {
                        foundUser(sumUser);
                    }
                    else {
                        anyPushedUser = true;
                        self.users()[sumUser.UserProviderKey] = ko.observable(sumUser);
                    }
                });

                if (anyPushedUser) // knockout nenotifikuje o nove pridanych uzivatelich
                    self.users(self.users());
                else
                    self.users.valueHasMutated();
            }

            if (summary.InitDateTime) // getSummary volani, melo by se to identifikovat nejak jinak
                self.users.extend({ rateLimit: 5000 });
        }

        var checkUpdates = self.chatItems().length > 0;
        var promises = [];
        if (summary.SecludedGuestUserSessions) {
            $(summary.SecludedGuestUserSessions).each(function (index, sumGuestUserSession) {
                if (self.selectedSessionId() === sumGuestUserSession.Session.SessionId) {
                    self.selectedSessionId(null);
                }

                processServerUserSession(sumGuestUserSession.Session.SessionId, sumGuestUserSession, promises, checkUpdates);
            });
        }

        if (summary.CoupledUserSessions) {
            for (var prop in summary.CoupledUserSessions) {
                if (summary.CoupledUserSessions.hasOwnProperty(prop)) {
                    processServerUserSession(prop, summary.CoupledUserSessions[prop], promises, checkUpdates);
                }
            }
        }

        if (promises.length > 0) {
            Promise.all(promises).then(function (values) {
                self.chatItems.valueHasMutated();
                selectDefaultChatItem();
            });
        }
        else {
            self.chatItems.valueHasMutated();
            selectDefaultChatItem();
        }
    };

    self.getMyExpression = function (session) {
        return self.getExpressionOfUser(session, userProviderKey);
    };

    self.getExpressionOfUser = function (session, userProviderKey) {
        let expression = session ? session.Expressions[userProviderKey] : null;
        if (expression)
            return expression;

        return null;
    };

    self.getAttitudeOfUser = function (session, userProviderKey) {
        let expression = self.getExpressionOfUser(session, userProviderKey);
        if ((expression === null || expression.Attitude === null && expression.Offer === null) && !session.WaitingMessage)
            return undefined;

        return expression ? expression.Attitude : null;
    };

    self.getMyAttitude = function (session) {
        return self.getAttitudeOfUser(session, userProviderKey);
    };

    self.isAcceptedByAnybody = function (session) {
        for (var prop in session.Expressions) {
            if (session.Expressions.hasOwnProperty(prop)) {
                if (session.Expressions[prop].Attitude === true)
                    return true;
            }
        }

        return false;
    };

    //operator only
    self.chatMessengerItems = ko.pureComputed(function () {
        return $.grep(self.chatItems(), function (chatItem) {
            return !chatItem.chatProfile && chatItem.myUserSession();
        });
    }, this);

    //operator only
    self.chatMessengerSightingItems = ko.pureComputed(function () {
        return $.grep(self.chatMessengerItems(), function (chatItem) {
            return chatItem.myUserSession().InSight;
        });
    }, this);

    //operator only
    self.chatGuestItems = ko.pureComputed(function () {
        return $.grep(self.chatItems(), function (chatItem) {
            return chatItem.chatProfile;
        }).sort(function (chatItem1, chatItem2) {
            return chatItem2.orderDate() - chatItem1.orderDate();
        });
    }, this);

    //operator only
    self.acceptedByAnybodyOrWaitingChatItems = ko.pureComputed(function () {
        return $.grep(self.chatGuestItems(), function (chatItem) {
            let guestUserSession = chatItem.guestUserSession();
            return guestUserSession && (self.isAcceptedByAnybody(guestUserSession.Session) === true || guestUserSession.Session.WaitingMessage);
        });
    }, this);

    //operator only
    self.acceptedByAnybodyChatItems = ko.pureComputed(function () {
        return $.grep(self.chatGuestItems(), function (chatItem) {
            let guestUserSession = chatItem.guestUserSession();
            return guestUserSession && self.isAcceptedByAnybody(guestUserSession.Session) === true;
        });
    }, this);

    //operator only
    self.waitingChatItems = ko.pureComputed(function () {
        return $.grep(self.chatGuestItems(), function (chatItem) {
            let guestUserSession = chatItem.guestUserSession();
            return guestUserSession && guestUserSession.Session.WaitingMessage;
        });
    }, this);

    //operator only
    self.acceptedUnspottedOrMyWaitingChatItems = ko.pureComputed(function () {
        if (self.operatorMode === false)
            return null;

        return $.grep(self.chatGuestItems(), function (chatItem) {
            let guestUserSession = chatItem.guestUserSession();
            if (guestUserSession)
            {
                let viewModel = chatItem.viewModel();
                let myExpression = self.getMyExpression(chatItem.session());
                let myAttitude = myExpression ? myExpression.Attitude : null;
                let isWaitingNotExpressedByMe = myAttitude === null && guestUserSession.Session.WaitingMessage && !chatUtils.chatItemIsBeyondChatProfileLimit(chatItem, self.chatItemsByChatProfile()[chatItem.chatProfile.Key]);
                let isOfferedNotExpressedByMe = myExpression !== null && myExpression.Offer !== null && myExpression.Attitude === null;
                return isWaitingNotExpressedByMe || isOfferedNotExpressedByMe || viewModel && viewModel.unspottedEventsCounter() > 0 && myExpression !== null && myExpression.Attitude === true;
            }

            return false;
        });
    }, this);

    self.activeUsers = ko.pureComputed(function () {
        let result = [];
        let users = self.users();
        if (users) {
            for (let key in users) {
                if (users.hasOwnProperty(key)) {
                    let user = users[key]();
                    if (user.IsActive && user.UserProviderKey !== userProviderKey) {
                        result.push(user);
                    }
                }
            }
        }

        return result.sort(function (leftUser, rightUser) {
            return leftUser.FullName === rightUser.FullName ? 0 : leftUser.FullName < rightUser.FullName ? -1 : 1;
        });
    }, this);

    self.virtualMessengerSessionsFilter = ko.observable('');
    self.virtualMessengerSessionsArray = ko.pureComputed(function () {
        let result = [];
        let lowerCaseVirtualMessengerSessionsFilter = self.virtualMessengerSessionsFilter().toLowerCase();
        for (let key in self.virtualMessengerSessions()) {
            if (self.virtualMessengerSessions().hasOwnProperty(key)) {
                let virtualSession = self.virtualMessengerSessions()[key];
                if (key !== userProviderKey && virtualSession.Operator.FullName.toLowerCase().includes(lowerCaseVirtualMessengerSessionsFilter))
                    result.push(virtualSession);
            }
        }

        let MIN_DATE = new Date(-8640000000000000);
        result = result.sort(function (s1, s2) {
            let lastMessageDate1 = s1.lastMessageDate() ? new Date(s1.lastMessageDate()) : new Date(MIN_DATE);
            let lastMessageDate2 = s2.lastMessageDate() ? new Date(s2.lastMessageDate()) : new Date(MIN_DATE);
            let result = lastMessageDate2.getTime() - lastMessageDate1.getTime();

            if (result === 0) {
                let users = self.users();
                let user1 = users && users[s1.Operator.UserProviderKey];
                let user2 = users && users[s2.Operator.UserProviderKey];
                let user1IsActive = user1 && user1().IsActive;
                let user2IsActive = user2 && user2().IsActive;
                result = (user2IsActive ? 1 : -1) - (user1IsActive ? 1 : -1);

                if (result === 0) {
                    result = s1.Operator.FullName.localeCompare(s2.Operator.FullName);
                }                
            }

            return result;
        });
        return result;
    }, this);

    //operator only
    self.acceptedUnspottedChatItems = ko.pureComputed(function () {
        return $.grep(self.chatGuestItems(), function (chatItem) {
            let viewModel = chatItem.viewModel();
            return viewModel && viewModel.unspottedEventsCounter() > 0 && self.getMyAttitude(chatItem.session()) === true;
        });
    }, this);

    //operator only
    self.activeGuestChatItems = ko.pureComputed(function () {
        return $.grep(self.chatGuestItems(), function (chatItem) {
            let guestUserSession = chatItem.guestUserSession();
            return guestUserSession && guestUserSession.IsActive;
        });
    }, this).extend({ rateLimit: 2000 });

    //operator only
    self.refuseSession = function (session, clearOffer, doneFunc) {
        chatHub.server.refuseSession(session.SessionId, clearOffer).done(function () {
            if (self.acceptedByAnybodyChatItems().length === 0)
                self.selectChatItem(null, false, false);

            if (typeof doneFunc === 'function')
                doneFunc(session);
        }).fail(function (error) {
            console.warn('Error refuseSession: ' + error);
        });
    };

    //operator only
    self.acceptSession = function (sessionId, clearOffer, doneFunc) {
        self.selectChatItem(sessionId, false, false);
        return new Promise(function (resolve, reject) {
            chatHub.server.acceptSession(sessionId, clearOffer).done(function () {
                if (typeof doneFunc === 'function')
                    doneFunc(sessionId);
                resolve();
            }).fail(function (error) {
                reject(error);
            });
        });
    };

    //operator only
    self.createGuestUserSessionAndAcceptIfNeeded = function (guestProviderKey, chatProfileKey) {
        return new Promise(function (resolve, reject) {
            chatHub.server.createGuestUserSessionAndAcceptIfNeeded(guestProviderKey, chatProfileKey).done(function (sessionId) {
                if (sessionId) {
                    self.selectChatItem(sessionId, false, false);
                }
                resolve(sessionId);
            }).fail(function (error) {
                console.warn('Error createGuestUserSessionAndAcceptIfNeeded: ' + error);
                reject(error);
            });
        });
    };

    self.validateAndSaveForm = function (formVals, sessionId) {
        return new Promise(function (resolve, reject) {
            chatHub.server.setFormValues(formVals, sessionId).done(function () {
                resolve();
            }).fail(function (error) {
                reject(error);
            });
        });
    };

    self.setChatProfileOperatorsAvailabilityBySession = function (operatorsAvailability, sessionId) {
        $(self.chatGuestItems()).each(function (index, chatItem) {
            var viewModel = chatItem.viewModel();
            if (viewModel && viewModel.isOnServer && viewModel.sessionId === sessionId) {
                viewModel.setChatProfileOperatorsAvailability(operatorsAvailability);
                return false;
            }
        });
    };

    self.setChatProfileOperatorsAvailabilityByChatProfile = function (operatorsAvailability, chatProfileKey) {
        $(self.chatGuestItems()).each(function (index, chatItem) {
            var viewModel = chatItem.viewModel();
            if (viewModel && !viewModel.isOnServer && viewModel.chatProfileKey === chatProfileKey) {
                viewModel.setChatProfileOperatorsAvailability(operatorsAvailability);
                return false;
            }
        });
    };

    self.addMessage = function (message) {
        $(self.chatItems()).each(function (index, chatItem) {
            let userSession = chatItem.myUserSession();
            if (userSession && userSession.Session.SessionId === message.SessionId)
            {
                chatItem.viewModel().addMessage(message);
                return false;
            }
        });
    };

    self.createMessengerUserSessionsIfNeeded = function (virtualMessengerSession) {
        if (virtualMessengerSession.chatItem())
            virtualMessengerSession.chatItem().isSelected(true);
        else
            virtualMessengerSession.selectionIsRequired = true;

        chatHub.server.createMessengerUserSessionsIfNeeded(virtualMessengerSession.Operator.UserProviderKey).fail(function (error) {
            console.warn('createMessengerUserSessionsIfNeeded: ' + error);
        });
    };

    self.getChatProfile = function (chatProfileKey) {
        if (self.chatProfiles) {
            for (let i = 0; i < self.chatProfiles.length; ++i) {
                let chatProfile = self.chatProfiles[i];
                if (chatProfile.Key === chatProfileKey) {
                    return chatProfile;
                }
            }
        }

        return null;
    };

    self.getChatProfileFromMessage = function (message) {
        for (var i = 0; i < self.chatGuestItems().length; ++i)
        {
            let chatItem = self.chatGuestItems()[i];
            if (chatItem.sessionId === message.SessionId) {
                return chatItem.chatProfile;
            }
        }

        return null;
    };

    //operator only
    self.loadOperatorsForSelectedSession = function () {
        self.clearOperatorsOfChatProfile();
        var selectedChatItem = self.selectedChatItem();
        if (selectedChatItem)
        {
            let chatProfileKey = selectedChatItem.chatProfile.Key;
            if (chatProfileKey)
            {
                $.getJSON(RQ_URL + "/Chat/GetAllOperatorsForChatProfileKey/" + chatProfileKey).done(function (res) {
                    
                    var operatorsOfChatProfile = [];

                    for (var prop in res) {
                        if (res.hasOwnProperty(prop) && prop !== userProviderKey && self.getAttitudeOfUser(selectedChatItem.session(), prop) !== true) {
                            operatorsOfChatProfile.push(new ChatOperator({
                                userProviderKey: prop,
                                fullName: res[prop]
                            }));
                        }
                    }

                    self.operatorsOfChatProfile(operatorsOfChatProfile);
                }).fail(function () {
                    self.operatorsOfChatProfile([]);
                    alert("Cannot load operators for chat profile.");
                });
            }
        }
    };

    //operator only
    self.clearOperatorsOfChatProfile = function () {
        self.selectedOperator(null);
        self.operatorsOfChatProfile(null);
    };

    //operator only
    self.addSelectedOperator = function () {
        chatHub.server.setOffer(self.selectedSessionId(), self.selectedOperator()).done(function () {

        }).fail(function (error) {
            console.warn('addSelectedOperator: ' + error);
        });
    };

    //operator only
    self.selectChatItem = function (sessionId, addUserSession, spot) {
        var previouslySelectedSessionId = addUserSession ? self.selectedSessionId() : undefined;
        self.selectedSessionId(sessionId);

        if (addUserSession || spot) {
            $(self.chatGuestItems()).each(function (index, chatItem) {
                if (chatItem.sessionId === sessionId)
                {
                    if (addUserSession && !chatItem.myUserSession())
                    {
                        chatHub.server.addUserSession(userProviderKey, sessionId).done(function () {
                            
                        }).fail(function (error) {
                            console.warn('addUserSession: ' + error);
                        });
                    }
                    else if (spot || previouslySelectedSessionId)
                    {
                        chatHub.server.spotUserSessions(spot ? sessionId : previouslySelectedSessionId, spot ? previouslySelectedSessionId : null).done(function () {
                            
                        }).fail(function (error) {
                            console.warn('spotUserSessions: ' + error);
                        });
                    }

                    return false;
                }
            });
        }
        
    };

    /**operaotr */
    self.exportSelectedChatSessionToPDF = function () {
        window.location.replace("Chat/ExportToPDF/" + self.selectedSessionId());
    };

    function acceptedByAnybodyChatItemsContainsSessionId(sessionId) {
        for (let i = 0; i < self.acceptedByAnybodyChatItems().length; ++i) {
            let chatItem = self.acceptedByAnybodyChatItems()[i];
            if (chatItem.sessionId === sessionId)
                return true;
        }

        return false;
    }

    function selectDefaultChatItem() {
        if (self.selectedSessionId()) {
            if (acceptedByAnybodyChatItemsContainsSessionId(self.selectedSessionId())) {
                return;
            }
        }

        if (selectedSessionId) {
            if (acceptedByAnybodyChatItemsContainsSessionId(selectedSessionId)) {
                self.selectChatItem(selectedSessionId, true, false);
                return;
            }
        }

        if (selectedSessionId !== undefined) {
            if (self.acceptedByAnybodyChatItems().length > 0) {
                self.selectChatItem(self.acceptedByAnybodyChatItems()[0].sessionId, true, false);
            }
                
        }
    }

    function startCurrentDateInterval(initDateTime) {
        var DATE_INTERVAL = 1000, oldInterval;
        var startIntervalDate = new Date();
        self.currentDate(new Date(initDateTime));
        var initDateTimeMiliseconds = self.currentDate().getTime();

        if (oldInterval)
            clearInterval(oldInterval);

        oldInterval = setInterval(function () {
            let spanMilliseconds = new Date() - startIntervalDate;
            let currentDate = new Date(initDateTimeMiliseconds + spanMilliseconds);
            self.currentDate(currentDate);
        }, DATE_INTERVAL);
    }

    function receiveCommonMessage(msgEnvelope) {
        $(self.chatItems()).each(function (index, chatItem) {
            let userSession = chatItem.myUserSession();
            if (userSession && userSession.Session.SessionId === msgEnvelope.sessionId) {
                chatItem.viewModel().receiveCommonMessage(msgEnvelope);
                return false;
            }
        });
    }

    self.awayCallback = function (autoAwayTimeout) {
        self.setAutoAwayStatus(autoAwayTimeout);
    };

    self.awayBackCallback = function () {
        self.setAutoAwayStatus(null);
    };

    self.getUsers = function () {
        if (self.users() !== null)
            return;

        self.isLoading(true);
        chatHub.server.getUsers().done(function (users) {
            if (users) {
                self.users({});
                $(users).each(function (index, user) {
                    self.users()[user.UserProviderKey] = ko.observable(user);
                });

                self.users(self.users());
                useStrippedSummary = false; //kdyz se bude znovu volat getSummary, tak uz plnohodnotne s uzivateli
            }
        }).fail(function (error) {
            console.error(error);
        }).always(function () {
            self.isLoading(false);
        });
    };

    var initialized = false;
    self.init = function (initialChatSessionViewModels) {
        if (initialized)
            return;

        initialized = true;
        chatHub = $.connection.chatHub;
        chatHub.client.setSummary = self.setSummary;
        chatHub.client.addMessage = self.addMessage;
        chatHub.client.receiveCommonMessage = receiveCommonMessage;
        if (!operatorMode) {
            chatHub.client.setChatProfileOperatorsAvailabilityByChatProfile = self.setChatProfileOperatorsAvailabilityByChatProfile;
            chatHub.client.setChatProfileOperatorsAvailabilityBySession = self.setChatProfileOperatorsAvailabilityBySession;
        }

        return new Promise(function (resolve, reject) {
            function addInitialChatSessionViewModels() {
                if (Array.isArray(initialChatSessionViewModels)) {
                    for (var i = 0; i < initialChatSessionViewModels.length; i++) {
                        let newChatItem = new ChatGuestItem(userProviderKey, self.currentDate, chatHub, initialChatSessionViewModels[i]);
                        newChatItem.init().then(function () {
                            self.chatItems().push(newChatItem);
                        });
                    }
                }
            }

            chatUtils.detectClientMedia().then(function (clientMedia) {
                var initialSessionIds = initialChatSessionViewModels ? {} : null;
                if (initialChatSessionViewModels) {
                    initialChatSessionViewModels.forEach(function (viewModel) {
                        initialSessionIds[viewModel.chatProfileKey] = viewModel.sessionId;
                    });
                }

                var resolveObj = {
                    initCallback: getSummary,
                    queryString: {
                        'userProviderKey': userProviderKey,
                        'sessionId': "",
                        'clientMedia': clientMedia,
                        'userAgent': JSON.stringify(new UAParser().getResult()),
                        'language': navigator.language || navigator.userLanguage,
                        'securedConnection': window.location.protocol === "https:",
                        'geo': chatUtils.loadGeoFromStorage(),
                        'initialSessionIds': initialSessionIds ? JSON.stringify(initialSessionIds) : null
                    }
                };

                if (operatorMode) {
                    let getJSONPromises = [];
                    getJSONPromises.push(
                        $.getJSON(RQ_URL + "/Chat/GetAllChatProfilesForOperator").done(function (chatProfiles) {
                            if (Array.isArray(chatProfiles) && chatProfiles.length > 0) {
                                self.chatProfiles = chatProfiles;
                                self.internalChatProfiles(chatProfiles.filter(function (chatProfile) {
                                    return chatProfile.IsInternal;
                                }));
                            }
                        })
                    );

                    getJSONPromises.push(
                        $.getJSON(RQ_URL + "/Chat/GetVirtualMessengerSessions").done(function (sessions) {
                            self.virtualMessengerSessions({});
                            if (Array.isArray(sessions) && sessions.length > 0) {
                                sessions.forEach(function (session) {
                                    session.chatItem = ko.observable();
                                    session.unspottedMessagesCount = ko.pureComputed(function () {
                                        return session.UnspottedMessagesCount = session.chatItem() ? !session.chatItem().isSelected() ? session.chatItem().viewModel().unspottedEventsCounter() : 0 : session.UnspottedMessagesCount;
                                    }, session);
                                    session.unspottedMessage = ko.pureComputed(function () {
                                        return session.LastUnspottedMessage = session.chatItem() ? !session.chatItem().isSelected() ? session.chatItem().viewModel().lastUnspottedMessage() : null : session.LastUnspottedMessage;
                                    }, session);
                                    session.lastMessageDate = ko.pureComputed(function () {
                                        return session.LastMessageDate = session.chatItem() ? session.chatItem().viewModel().lastMessageDate() : session.LastMessageDate;
                                    }, session);
                                    session.lastActivity = ko.pureComputed(function () {
                                        let user = self.users() && self.users()[session.Operator.UserProviderKey];
                                        let lastActivity = session.LastActivity = !user || (!user().IsActive || user().Status === chatUtils.ChatUserStatusEnum.AWAY) ? user ? user().LastActivity : session.LastActivity : null;
                                        return lastActivity ? new Date(lastActivity) : null;
                                    }, session);
                                    session.callOptions = ko.pureComputed(function () {
                                        return session.chatItem() ? session.chatItem().viewModel().callOptions() : null;
                                    }, session);

                                    self.virtualMessengerSessions()[session.Operator.UserProviderKey] = session;
                                });
                            }
                        })
                    );

                    Promise.all(getJSONPromises).then(function (values) {
                        addInitialChatSessionViewModels();
                        resolve(resolveObj);
                    }).catch(function (error) {
                        reject(error);
                    });
                   
                }
                else {
                    addInitialChatSessionViewModels();
                    resolve(resolveObj);
                }
            }).catch(function (error) {
                reject(error);
            });
        });

        function getSummary() {
            return new Promise(function (resolve, reject) {
                self.isLoading(true);

                self.users(null);
                var notOnServerChatItems = [];
                self.chatItems().forEach(function (chatItem) {
                    if (chatItem.viewModel() && !chatItem.viewModel().isOnServer) {
                        notOnServerChatItems.push(chatItem);
                    }
                });

                self.chatItems.removeAll();
                ko.utils.arrayPushAll(self.chatItems, notOnServerChatItems);

                self.userStatus(null);
                self.canReceiveNewGuests(null);
                self.currentDate(undefined);
                self.operatorsOfChatProfile([]);
                self.selectedOperator(null);

                //ziskavani noveho summmary
                chatHub.server.getSummary2(useStrippedSummary).done(function (summary) {
                    if (summary) {
                        startCurrentDateInterval(summary.InitDateTime);
                        self.setSummary(summary);
                    }

                    self.isLoading(false);
                    resolve();
                }).fail(function (error) {
                    self.isLoading(false);
                    reject(error);
                });
            });
        }
    };

    //init
    var chatHub;
}